<?php

namespace Bixo\Deal\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Bixo\Deal\Events\ContactAction as ContactActionEvent;
use Bixo\Deal\Notifications\ContactAction as ContactActionNotification;
use Litepie\Actions\Concerns\AsAction;

class ContactAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the ContactActionEvent.
     *
     * @param   ContactActionEvent  $event
     * @return mixed
     */
    public function handle(ContactActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $contact.
     *
     * @param   ContactActionEvent  $event
     * @return void
     */
    public function create(ContactActionEvent $event)
    {
        $client = $event->contact->client;
        Notification::send($client, new ContactActionNotification($event));
    }

    /**
     * Handle the ContactActionEvent as a listener.
     *
     * @param   ContactActionEvent  $event
     * @return mixed
     */
    public function asListener(ContactActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   ContactActionEvent  $event
     * @return bool
     */
    private function isAllowed(ContactActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
