<?php

namespace Bixo\Deal\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Bixo\Deal\Events\DealAction as DealActionEvent;
use Bixo\Deal\Notifications\DealAction as DealActionNotification;
use Litepie\Actions\Concerns\AsAction;

class DealAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the DealActionEvent.
     *
     * @param   DealActionEvent  $event
     * @return mixed
     */
    public function handle(DealActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $deal.
     *
     * @param   DealActionEvent  $event
     * @return void
     */
    public function create(DealActionEvent $event)
    {
        $client = $event->deal->client;
        Notification::send($client, new DealActionNotification($event));
    }

    /**
     * Handle the DealActionEvent as a listener.
     *
     * @param   DealActionEvent  $event
     * @return mixed
     */
    public function asListener(DealActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   DealActionEvent  $event
     * @return bool
     */
    private function isAllowed(DealActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
