<?php

namespace Bixo\Deal\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Bixo\Deal\Events\DealWorkflow as DealWorkflowEvent;
use Bixo\Deal\Notifications\DealWorkflow as DealWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class DealWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(DealWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $deal is submitted.
     *
     * @param DealWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(DealWorkflowEvent $event)
    {
        $client = $event->deal->client;
        Notification::send($client, new DealWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $deal is published.
     *
     * @param DealWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(DealWorkflowEvent $event)
    {
        $client = $event->deal->client;
        Notification::send($client, new DealWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $deal is approved.
     *
     * @param DealWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(DealWorkflowEvent $event)
    {
        $client = $event->deal->client;
        Notification::send($client, new DealWorkflowNotification($event));
    }

    /**
     * Handles the $deal workflow event as a listener.
     *
     * @param DealWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(DealWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
