<?php
namespace Bixo\Deal\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Bixo\Deal\Events\DealAction as DealActionEvent;
use Bixo\Deal\Models\Deal;

class DealAction extends Notification implements ShouldQueue
{
    use Queueable;

    protected $event;
    protected $deal;
    protected $action;
    protected $request;

    /**
     * Create a new notification instance.
     *
     * @param DealActionEvent $deal
     */
    public function __construct(DealActionEvent $deal)
    {
        $this->action = $event->action;
        $this->deal = $deal;
        $this->request = $event->request;
        $this->event = $event;

    }

    /**
     * Get the mail representation of the notification.
     *
     * @param mixed $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {
        return (new MailMessage)
            ->subject('Deal Approved - ' . config('app.name'))
            ->view('deal::deal.email.action', ['data' => $this->deal]);
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param object $notifiable
     * @return array<int, string>
     */
    public function via(object $notifiable): array
    {
        return ['mail'];
    }
}
