<?php

namespace Bixo\Deal\Policies;

use Illuminate\Foundation\Auth\User as Authenticatable;
use Bixo\Deal\Models\Contact;

class ContactPolicy
{


    /**
     * Determine if the given user can view the contact.
     *
     * @param Authenticatable $user
     * @param Contact $contact
     *
     * @return bool
     */
    public function view(Authenticatable $user, Contact $contact)
    {
        if ($authUser->canDo('deal.contact.view') && $authUser->isAdmin() || $user->isClient()) {
            return true;
        }

        return $contact->is_owner;
    }

    /**
     * Determine if the given user can create a contact.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function create(Authenticatable $user)
    {
        return  $authUser->canDo('deal.contact.create');
    }

    /**
     * Determine if the given user can update the given contact.
     *
     * @param Authenticatable $user
     * @param Contact $contact
     *
     * @return bool
     */
    public function update(Authenticatable $user, Contact $contact)
    {
        if ($user->canDo('deal.contact.edit') && $user->isAdmin()) {
            return true;
        }

        return $contact->is_owner;
    }

    /**
     * Determine if the given user can delete the given contact.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function destroy(Authenticatable $user, Contact $contact)
    {
        return $contact->is_owner;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
