<?php

namespace Bixo\Deal\Policies;

use Illuminate\Foundation\Auth\User as Authenticatable;
use Bixo\Deal\Models\Deal;

class DealPolicy
{

    use DealAction;
    use DealWorkflow;

    /**
     * Determine if the given user can view the deal.
     *
     * @param Authenticatable $user
     * @param Deal $deal
     *
     * @return bool
     */
    public function view(Authenticatable $user, Deal $deal)
    {
        if ($authUser->canDo('deal.deal.view') && $authUser->isAdmin() || $user->isClient()) {
            return true;
        }

        return $deal->is_owner;
    }

    /**
     * Determine if the given user can create a deal.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function create(Authenticatable $user)
    {
        return  $authUser->canDo('deal.deal.create');
    }

    /**
     * Determine if the given user can update the given deal.
     *
     * @param Authenticatable $user
     * @param Deal $deal
     *
     * @return bool
     */
    public function update(Authenticatable $user, Deal $deal)
    {
        if ($user->canDo('deal.deal.edit') && $user->isAdmin()) {
            return true;
        }

        return $deal->is_owner;
    }

    /**
     * Determine if the given user can delete the given deal.
     *
     * @param Authenticatable $user
     *
     * @return bool
     */
    public function destroy(Authenticatable $user, Deal $deal)
    {
        return $deal->is_owner;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
