<?php

namespace Bixo\Deal\Providers;

use Illuminate\Support\ServiceProvider;
use Bixo\Deal\Deals;

class DealServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = true;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'deal');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'deal');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerFacade();

        $this->app->register(\Bixo\Deal\Providers\AuthServiceProvider::class);
        $this->app->register(\Bixo\Deal\Providers\RouteServiceProvider::class);
        $this->app->register(\Bixo\Deal\Providers\ActionServiceProvider::class);
        $this->app->register(\Bixo\Deal\Providers\WorkflowServiceProvider::class);
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->bind('bixo.deal', function($app)
        {
            return $this->app->make(Deal::class);
        });
    }

    /**
     * Merges user's and deal's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'bixo.deal'
        );
        
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/deal.php', 'bixo.deal.deal'
        );
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/contact.php', 'bixo.deal.contact'
        );
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/user.php', 'bixo.deal.user'
        );
    }


    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['bixo.deal'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/' => config_path('bixo/deal')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/deal')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/deal')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
